@@ -1,4 +1,4 @@
-use crate::{Mutex, MutexSized, TryMutex, TryMutexSized};
+use crate::mutex::{Mutex, MutexSized, TryMutex, TryMutexSized};
 use std::sync;
 use std::sync::{MutexGuard, TryLockError};
 
