@@ -21,13 +21,9 @@ where
 {
     /// Creates a new [`AsyncCustomQueue`] from a backing queue and message
     /// queue
-    pub fn new<TS>(
-        queue: Q,
-        message_queue: MQ,
-        thread_spawner: TS,
-    ) -> Result<(Self, TS::SpawnReturn), TS::SpawnError>
+    pub fn new<TS>(queue: Q, message_queue: MQ) -> Result<(Self, TS::ThreadHandle), TS::SpawnError>
     where
-        TS: ThreadSpawner,
+        TS: ThreadSpawner<()>,
     {
         let inner = Arc::new(AsyncQueueInner {
             queue,
@@ -36,7 +32,7 @@ where
         let weak_inner = Arc::downgrade(&inner);
         Ok((
             Self { inner },
-            thread_spawner.spawn(move || Self::queue_task_function(weak_inner))?,
+            TS::try_spawn(move || Self::queue_task_function(weak_inner))?,
         ))
     }
 
@@ -136,13 +132,9 @@ where
 {
     /// Creates a new [`AsyncCustomPrependQueue`] from a backing queue and
     /// message queue
-    pub fn new<TS>(
-        queue: Q,
-        message_queue: MQ,
-        thread_spawner: TS,
-    ) -> Result<(Self, TS::SpawnReturn), TS::SpawnError>
+    pub fn new<TS>(queue: Q, message_queue: MQ) -> Result<(Self, TS::ThreadHandle), TS::SpawnError>
     where
-        TS: ThreadSpawner,
+        TS: ThreadSpawner<()>,
     {
         let inner = Arc::new(AsyncQueueInner {
             queue,
@@ -151,7 +143,7 @@ where
         let weak_inner = Arc::downgrade(&inner);
         Ok((
             Self { inner },
-            thread_spawner.spawn(move || Self::queue_task_function(weak_inner))?,
+            TS::try_spawn(move || Self::queue_task_function(weak_inner))?,
         ))
     }
 
@@ -279,13 +271,9 @@ where
 {
     /// Creates a new [`AsyncCustomReverseQueue`] from a backing queue and
     /// message queue
-    pub fn new<TS>(
-        queue: Q,
-        message_queue: MQ,
-        thread_spawner: TS,
-    ) -> Result<(Self, TS::SpawnReturn), TS::SpawnError>
+    pub fn new<TS>(queue: Q, message_queue: MQ) -> Result<(Self, TS::ThreadHandle), TS::SpawnError>
     where
-        TS: ThreadSpawner,
+        TS: ThreadSpawner<()>,
     {
         let inner = Arc::new(AsyncQueueInner {
             queue,
@@ -294,7 +282,7 @@ where
         let weak_inner = Arc::downgrade(&inner);
         Ok((
             Self { inner },
-            thread_spawner.spawn(move || Self::queue_task_function(weak_inner))?,
+            TS::try_spawn(move || Self::queue_task_function(weak_inner))?,
         ))
     }
 
@@ -415,13 +403,9 @@ where
 {
     /// Creates a new [`AsyncCustomDoubleEndedQueue`] from a backing queue and
     /// message queue
-    pub fn new<TS>(
-        queue: Q,
-        message_queue: MQ,
-        thread_spawner: TS,
-    ) -> Result<(Self, TS::SpawnReturn), TS::SpawnError>
+    pub fn new<TS>(queue: Q, message_queue: MQ) -> Result<(Self, TS::ThreadHandle), TS::SpawnError>
     where
-        TS: ThreadSpawner,
+        TS: ThreadSpawner<()>,
     {
         let inner = Arc::new(AsyncQueueInner {
             queue,
@@ -430,7 +414,7 @@ where
         let weak_inner = Arc::downgrade(&inner);
         Ok((
             Self { inner },
-            thread_spawner.spawn(move || Self::queue_task_function(weak_inner))?,
+            TS::try_spawn(move || Self::queue_task_function(weak_inner))?,
         ))
     }
 
